# in spec/calculator_spec.rb
# - RSpec adds ./lib to the $LOAD_PATH
require "pgm/model"

RSpec.describe PGM::BayesianNet do

  context "test graph module" do
    def build_model
      PGM::BayesianNet[*'ACCFBDBEDFEHFHFG'.chars]
    end

    before(:example) do
      @model = build_model
    end

    describe '#common_parents' do
      it 'returns the blank array' do
        expect(@model.common_parents).to eq([])
      end

      it 'returns the common parent list' do
        expect(@model.common_parents(*'GH'.chars)).to eq(['F'])
      end
    end

    describe '#common_parents?' do
      it 'returns false when blank input' do
        expect(@model.common_parents?).to eq(false)
      end

      it 'returns false when no common parents' do
        expect(@model.common_parents?(*'CD'.chars)).to eq(false)
        expect(@model.common_parents?(*'CDE'.chars)).to eq(false)
      end

      it 'returns true when there are common parents' do
        expect(@model.common_parents?(*'GH'.chars)).to eq(true)
      end
    end

    describe '#common_children' do
      it 'returns the blank array' do
        expect(@model.common_children).to eq([])
      end

      it 'returns the common child list' do
        expect(@model.common_children(*'CD'.chars)).to eq(['F'])
      end
    end

    describe '#common_children?' do
      it 'returns false when blank input' do
        expect(@model.common_children?).to eq(false)
      end

      it 'returns false when no common children' do
        expect(@model.common_children?(*'CE'.chars)).to eq(false)
        expect(@model.common_children?(*'CDE'.chars)).to eq(false)
      end

      it 'returns true when there are common children' do
        expect(@model.common_children?(*'CD'.chars)).to eq(true)
      end
    end

    describe '#cascade?' do
      it 'returns false when blank inputs' do
        expect(@model.cascade?).to eq(false)
      end

      it 'returns false when not cascading' do
        expect(@model.cascade?(*'AC'.chars)).to eq(false)
        expect(@model.cascade?(*'ACH'.chars)).to eq(false)
        expect(@model.cascade?(*'ADE'.chars)).to eq(false)
      end

      it 'returns true when cascading' do
        expect(@model.cascade?(*'ACF'.chars)).to eq(true)
        expect(@model.cascade?(*'ACFH'.chars)).to eq(true)
      end
    end

    describe '#v_tructure?' do
      it 'returns false when wrong number of node' do
        expect(@model.v_tructure?).to eq(false)
        expect(@model.v_tructure?(*'AC'.chars)).to eq(false)
        expect(@model.v_tructure?(*'FCDB'.chars)).to eq(false)
      end

      it 'returns false when not in v-structure' do
        expect(@model.v_tructure?(*'CFD'.chars)).to eq(false)
        expect(@model.v_tructure?(*'BDE'.chars)).to eq(false)
        expect(@model.v_tructure?(*'FAB'.chars)).to eq(false)
      end

      it 'returns true when v-structure' do
        expect(@model.v_tructure?(*'FCD'.chars)).to eq(true)
      end
    end

    describe '#model_notation' do
      it 'returns a joint ditribution string' do
        expect(@model.model_notation).to eq('P(A,B,C,D,E,F,G,H) = P(A)P(B)P(C|A)P(D|B)P(E|B)P(F|C,D)P(G|F)P(H|E,F)')
      end
    end
  end

  context "testing probability net" do
    def build_model
      a = PGM::BayesianNet.new()
      a.create_var(:cloudy, [:t, :f])
      a.create_var(:sprinkler, [:t, :f])
      a.create_var(:rain, [:t, :f])
      a.create_var(:grass_wet, [:t, :f])
      a.create_link(:cloudy, :sprinkler)
      a.create_link(:cloudy, :rain)
      a.create_link(:sprinkler, :grass_wet)
      a.create_link(:rain, :grass_wet)
      return a
    end

    before(:example) do
      @model = build_model
    end

    describe '#model_notation' do
      it 'returns a model notation string' do
        expect(@model.model_notation).to eq("P(cloudy,grass_wet,rain,sprinkler) = P(cloudy|sprinkler,rain)P(grass_wet)P(rain|grass_wet)P(sprinkler|grass_wet)")
      end
    end

    describe '#learning' do
      it 'returns a joint ditribution string' do
        @model.learn([
          {:cloudy => :t, :sprinkler => :f, :rain => :t, :grass_wet => :t},
          {:cloudy => :t, :sprinkler => :t, :rain => :f, :grass_wet => :t},
          {:cloudy => :f, :sprinkler => :f, :rain => :t, :grass_wet => :t},
          {:cloudy => :t, :sprinkler => :f, :rain => :t, :grass_wet => :t},
          {:cloudy => :f, :sprinkler => :t, :rain => :f, :grass_wet => :f},
          {:cloudy => :f, :sprinkler => :f, :rain => :f, :grass_wet => :f},
          {:cloudy => :f, :sprinkler => :f, :rain => :f, :grass_wet => :f},
          {:cloudy => :t, :sprinkler => :f, :rain => :t, :grass_wet => :t},
          {:cloudy => :t, :sprinkler => :f, :rain => :f, :grass_wet => :f},
          {:cloudy => :f, :sprinkler => :f, :rain => :f, :grass_wet => :f},
        ])
        expect(@model.var(:cloudy).conditional_probability({:cloudy => :f}).to_a).to eq([0.75, 0.4999999403953552, 0.25, 0.0])
      end
    end
  end
end
